<?php
require_once ASD_APPLICATION_PATH . '/models/AbstractModel.php';

class Asd_Model_Amazon_Search extends Asd_Model_AbstractModel
{
    const PREFER_DESCRIPTION_TYPE = 'Product Description';
    const MIN_DESCRIPTION_LENGTH = 500;

    protected $_maxResultCount = 10;

    public function search(array $options)
    {
        $amazon = Asd_Application::getModel('Amazon')
            ->setOption('market', $options['market']);
        if (isset($options['affiliate_id'])) {
            $amazon->setOption('affiliate_id', $options['affiliate_id']);
        }

        // Compose rh parameter

        $rh = array();

        $browseNode = NULL;
        if (isset($options['subcategory']) && '' != $options['subcategory']) {
            $browseNode = $options['subcategory'];
        } else if (isset($options['category']) && '' != $options['category']) {
            $browseNode = $options['category'];
        }
        if ($browseNode) {
            $rh['n'] = $browseNode;
        } else {
            $rh['i'] = 'aps';
        }

        if (isset($options['discount']) && '' != $options['discount']) {
            $rh['p_8'] = $options['discount'];
        }

        $rh['k'] = '*';
        if (isset($options['query']) && '' != $options['query']) {
            $rh['k'] = $options['query'];
        }

        foreach ($rh as $key=>&$val) {
            $val = "{$key}:{$val}";
        }
        unset($val);
        $rh = implode(',', $rh);

        $params = array(
            'rh' => $rh,
            'ie' => 'UTF8',
            'qid' => time(),
            'section' => 'ATF,BTF',
            'fromApp' => 'gp/search',
            'fromPage' => 'results',
            'version' => '2',
        );

        try {
            $response = $amazon->getHttp()
                ->setUri('http://www.' . $amazon->getMarketConfig()->domain . '/mn/search/ajax/ref=nb_sb_noss')
                ->setParameterGet($params)
                ->request('GET');
        } catch (AsdZend_Http_Client_Exception $e) {
            $this->_throwError(sprintf(__('Could not connect to Amazon search API (%s)', 'amazon-bar-plugin'), $e->getMessage()));
        } catch (Exception $e) {
            $this->_throwError(sprintf(__('Unexpected error while requesting Amazon search API (%s)', 'amazon-bar-plugin'), $e->getMessage()));
        }
        if ($response->isError()) {
            $this->_throwError(sprintf(__('Amazon search API request failed due to HTTP error #%s (%s)', 'amazon-bar-plugin'), $response->getStatus(), $e->getMessage()));
        }

        $response = $response->getBody();

        if (preg_match('~"results-atf" : {(.+?)"tagId" : "btfResults"~isu', $response, $matches)) {
            $response = $matches[1];
        } else if (preg_match('~"results-btf" : {(.+?)"tagId" : "btfResults"~isu', $response, $matches)) {
            $response = $matches[1];
        }

        $maxCount = Asd_Application::getModel('Options')
            ->getValue('max_product_count');

        $result = array();

        $fragments = preg_split('~<div id=\\\\"result_.*?~isu', $response);
        foreach ($fragments as $fragment) {
            if (!preg_match('~^[^>]*class=\\\\"[^"]*prod.*?\\\\".*?\bname=\\\\"(.+?)\\\\"~isu',
                $fragment, $matches)
            ) {
                continue;
            }

            $asin = $matches[1];

            $itemData = $amazon->getItemData($asin);

            if (!$itemData) {
                continue;
            }
            if (!$amazon->isItemAvailable($itemData)) {
                continue;
            }

            if (!isset($itemData['EditorialReviews'])) {
                continue;
            }

            $itemData['etc'] = array();

            $description = $this->_extractDescription($itemData);
            if ('' == $description) {
                continue;
            }
            $itemData['etc']['description'] = $description;

            if (preg_match('~(\d(?:\.\d)?) (?:out of 5 stars|von 5 Sternen|de un máximo de 5 estrellas)~isu',
                $fragment, $matches)
            ) {
                $rating = (double) $matches[1];
                if ($rating) {
                    $itemData['etc']['rating'] = $rating;
                }
            }

            $result[$asin] = $itemData;

            if (count($result) >= $this->_maxResultCount) {
                break;
            }
        }

        return $result;
    }

    protected function _throwError($message)
    {
        require_once ASD_APPLICATION_PATH . '/models/Amazon/Exception.php';
        throw new Asd_Model_Amazon_Exception($message);
    }

    protected function _extractDescription($itemData)
    {
        if (!isset($itemData['EditorialReviews']['EditorialReview'])) {
            return NULL;
        }

        $items = $itemData['EditorialReviews']['EditorialReview'];
        if (!isset($items[0])) {
            // Single item
            $items = array($items);
        }

        $result = NULL;
        foreach ($items as $review) {
            $current = $review['Content'];
            // Prefer Product Description
            if (self::PREFER_DESCRIPTION_TYPE == $review['Source']
                && iconv_strlen($result) >= self::MIN_DESCRIPTION_LENGTH
            ) {
                $result = $current;
                break;
            }

            if (iconv_strlen($current) > iconv_strlen($result)) {
                $result = $current;
            }
        }

        if ('' != $result) {
            $result = Asd_Application::getModel('Helper_Text')
                ->htmlFragmentToText($result);
        }

        return $result;
    }
}