/**
* Ama Search Bar admin JS
*/
"use strict";

jQuery(function($) {

    //
    // Plugins
    //

    $.fn.setDisabled = function(value) {
        if (typeof this['prop'] === 'function') {
            this.prop('disabled', value);
        } else if (value) {
            this.attr('disabled', 'disabled');
        } else {
            this.removeAttr('disabled');
        }

        if (value) {
            this.addClass('disabled');
        } else {
            this.removeClass('disabled');
        }

        return this;
    };

    $.fn.resetCategorySelect = function(){
        this.empty().append('<option value="0">' + asdLocaleAdmin.anyPlaceholder + '</option>');

        return this;
    };

    $.fn.waitCategorySelect = function(){
        this.empty()
            .setDisabled(true)
            .append('<option>' + asdLocaleAdmin.loading + '</option>');

        return this;
    };

    //
    // Common init
    //

    $("#asd-affiliate_id-dialog").dialog({
        autoOpen: false,
        dialogClass: 'wp-dialog asd-dialog',
        modal: true,
        buttons: [{text: asdLocaleAdmin.ok, click: function(){ $(this).dialog('close'); }}],
        draggable: false,
        resizable: false,
        //title: asdLocaleAdmin.dialogTitleAssociateIds
        create: function () {
            $(this).siblings().find(".ui-dialog-title").html(asdLocaleAdmin.dialogTitleAssociateIds);
        }
    });
    $("#asd-affiliate_id-button").click(function() {
        $("#asd-affiliate_id-dialog").dialog('open');
    });

    $("#asd-category-dialog").dialog({
        autoOpen: false,
        dialogClass: 'wp-dialog asd-dialog',
        modal: true,
        buttons: [{text: "OK", click: function(){ $(this).dialog('close'); }}],
        draggable: false,
        resizable: false,
        //title: asdLocaleAdmin.dialogTitleCategories,
        width: 510,
        create: function () {
            $(this).siblings().find(".ui-dialog-title").html(asdLocaleAdmin.dialogTitleCategories);
        }
    });
    $("#asd-category-button").click(function() {
        $("#asd-category-dialog").dialog('open');
    });

    $('.asd-credential-field').change(function() {
        var hasMainCredentials = '' != $.trim($('#access_key').val())
            && '' != $.trim($('#secret_access_key').val());
        for (var i in asd.markets) {
            var marketCode = asd.markets[i];
            if ('function' == typeof(marketCode)) {
                continue;
            }

            var enabled = hasMainCredentials;
            if (enabled) {
                enabled = '' != $.trim($('#' + marketCode + '_affiliate_id').val());
            }

            var categoryElem = $('#' + marketCode + '_category');
            var subcategoryElem = $('#' + marketCode + '_subcategory');
            categoryElem.setDisabled(!enabled);
            subcategoryElem.setDisabled(!enabled || '0' == categoryElem.val());
        }
    });

    //
    // Common event handlers
    //

    $('.asd-form-wrap .market').change(function() {
        setDisalogsMarket($(this).val());
    });

    $('.asd-form-wrap .category').change(function() {
        var value = $(this).val();
        var marketCode = $(this).attr('data-market');

        var subcategory = $('.asd-form-wrap .' + marketCode + '_subcategory');


        if ('0' == value) {
            subcategory.resetCategorySelect()
                .setDisabled(true);

            return;
        }

        subcategory.waitCategorySelect();

        $.ajax({
            type: 'POST',
            url: ajaxurl,
            dataType: 'json',
            data: {
                action: 'asd',
                controller: 'admin-ajax',
                controller_action: 'get-categories',
                params: {
                    market: marketCode,
                    parent: value,
                    options: collectAmazonOptions(marketCode)
                }
            },
            context: subcategory,
            success: function(data) {
                this.resetCategorySelect();
                for (var i in data) {
                    var item = data[i]
                    if ('function' == typeof item) {
                        // Weird bug
                        continue;
                    }

                    this.append(jQuery('<option/>', {
                        value: item['id'],
                        text: item['label']
                    }));
                }
            },
            error: function(xhr) {
                this.resetCategorySelect();
                handleAjaxError(xhr);
            },
            complete: function() {
                this.setDisabled(false);
            }
        });
    });

    $('.asd-form-wrap .has-colorpicker').each(function() {
        $('#' + $(this).attr('id') + '-colorpicker').farbtastic('#' + $(this).attr('id'));

        $(this).focus(function() {
            $('#' + $(this).attr('id') + '-colorpicker').slideDown();
        });

        $(this).focusout(function() {
            $('#' + $(this).attr('id') + '-colorpicker').slideUp();
        });
    });

    $('#asd-form-admin').submit(function() {
        $('#dialog_data').val($('.asd-dialog-content input, .asd-dialog-content select, .asd-dialog-content textarea').serialize());
    });

    //
    // Admin page functions
    //

    function collectAmazonOptions(marketCode)
    {
        var result = {};
        $('.asd-form-wrap .asd-amazon-option-field').each(function() {
            result[$(this).attr('data-amazon-option')] = $(this).val();
        });
        result['market'] = marketCode;

        return result;
    }

    //
    // Metabox
    //

    $('#asd-form-metabox #asd_metabox-customize_categories').change(function() {
        if ($(this).attr('checked')) {
            $('.asd-form-metabox .category-field').each(function()  {
                if ($(this).hasClass('subcategory')
                    && $('option', this).length <= 1
                ) {
                    $(this).setDisabled(true);
                } else {
                    $(this).setDisabled(false);
                }
            });
        } else {
            $('.asd-form-metabox .category-field').addClass('disabled');
        }
    });
    $('.asd-form-metabox .category-field').click(function() {
        if ($(this).hasClass('disabled')) {
            $('#asd-form-metabox #asd_metabox-customize_categories')
                .attr('checked', 'checked')
                .change();
        }
    });

    $('#asd-form-metabox .button_update').click(function() {
        this.disabled = true;
        $(this).addClass('button-primary-disabled');
        $('#asd-metabox-ajax_loader').css('visibility', 'visible');
        blockSave = true;
        window.onbeforeunload = null;
        $('#post').submit();
    });

    $('#post').submit(function() {
        $('#asd_metabox-dialog_data').val($('.asd-dialog-content input, .asd-dialog-content select, .asd-dialog-content textarea').serialize());
    });

    //
    // Common functions
    //

    function handleAjaxError(xhr) {
        if (500 != xhr.status) {
            alert(asdLocaleAdmin.ajaxError);
        } else if (null != xhr.getResponseHeader('X-Ajax-Error')) {
            alert(xhr.responseText);
        } else if (null != xhr.getResponseHeader('X-Ajax-Error-Unhandled')) {
            alert(asdLocaleAdmin.pluginError);
        } else {
            alert(asdLocaleAdmin.serverError);
        }
    }

    function setDisalogsMarket(market)
    {
        //
        // affiliate_id dialog
        //

        $('#asd-affiliate_id-dialog .asd-form-field').each(function() {
            $(this).toggle(market != $(this).attr('data-market'));
        });

        var elem = $('.asd-form-main .affiliate_id-container .asd-input');
        if (elem.length) {
            $('#asd-affiliate_id-dialog .' + elem.attr('data-market') + '_affiliate_id-container')
                .append(elem);
        }

        var elem = $('#asd-affiliate_id-dialog .' + market + '_affiliate_id');
        $('.asd-form-main .affiliate_id-container').append(elem);
        $('.asd-form-main .affiliate_id-label').attr('for', elem.attr('id'));

        //
        // category  dialog
        //

        $('#asd-category-dialog .widefat tbody tr').each(function() {
            $(this).toggle(market != $(this).attr('data-market'));
        });

        var elem = $('.asd-form-main .category-container .asd-input');
        if (elem.length) {
            $('#asd-category-dialog .' + elem.attr('data-market') + '_category-container')
                .append(elem);
        }
        var elem = $('.asd-form-main .subcategory-container .asd-input');
        if (elem.length) {
            $('#asd-category-dialog .' + elem.attr('data-market') + '_subcategory-container')
                .append(elem);
        }

        var elem = $('#asd-category-dialog .' + market + '_category');
        $('.asd-form-main .category-container').append(elem);
        $('.asd-form-main .category-label').attr('for', elem.attr('id'));
        var elem = $('#asd-category-dialog .' + market + '_subcategory');
        $('.asd-form-main .subcategory-container').append(elem);
        $('.asd-form-main .subcategory-label').attr('for', elem.attr('id'));
    }
});